#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <utime.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "FastMake.h"
#include "file.h"
#include "search.h"
#include "string.h"
#include "xml.h"

int main(int argc, char **argv)
{
    xmlDocPtr doc;
    xmlNodePtr element_root;

    xmlNodePtr element_project;
    xmlNodePtr element_system;

    if (!(doc = xmlParseFile(argv[1]))) {
	perror("xmlParseFile");
	exit(0);
    }
    if (!(element_root = xmlDocGetRootElement(doc))) {
	perror("xmlGetRootElement");
	exit(0);
    }
    if (xmlStrcmp(element_root->name, (const xmlChar *) "FastMake")) {
	printf("ERRO: %s - Missing root_element(FastMake) - Found: (%s)\n", argv[1], element_root->name);
	fflush(stdout);
	exit(0);
    }

    element_project = getElement(doc, element_root->xmlChildrenNode, "project");
    element_system = getElement(doc, element_root->xmlChildrenNode, "system");

    build_project(doc, element_system, element_project);

    return 0;
}

int build_project(xmlDocPtr doc, xmlNodePtr element_system, xmlNodePtr element_project)
{
    char *args = NULL;
    xmlNodePtr element_program;
    xmlNodePtr element_install = NULL;
    xmlNodePtr element_exec = NULL;

    printf("Building project: %s\n", xmlGetProp(element_project, "name"));

    element_program = element_project->xmlChildrenNode;
    while ((element_program = getElement(doc, element_program, "program"))) {
	build_program(doc, element_system, element_program, element_project);

	element_program = element_program->next;
    }



    element_install = element_project->xmlChildrenNode;
    while ((element_install = getElement(doc, element_install, "install"))) {
	makedir(xmlGetProp(element_project, "default_prefix"), xmlGetProp(element_install, "directory"));


	args = str_add(args, "cp -f ");
	args = str_add(args, xmlGetProp(element_install, "file"));
	args = str_add(args, " ");
	if (xmlGetProp(element_install, "directory")[0] != '/') {
	    args = str_add(args, xmlGetProp(element_project, "default_prefix"));
	    args = str_add(args, "/");
	}
	args = str_add(args, xmlGetProp(element_install, "directory"));
	printf("%s\n", args);
	run_command(args);

	free(args);
	args = NULL;

	element_install = element_install->next;
    }

/* EXEC */
    element_exec = element_project->xmlChildrenNode;
    while ((element_exec = getElement(doc, element_exec, "exec"))) {
	args = str_add(args, xmlGetProp(element_exec, "program"));
	printf("%s\n", args);
	system(args);

	free(args);
	args = NULL;

	element_exec = element_exec->next;
    }

    return 1;
}

int build_program(xmlDocPtr doc, xmlNodePtr element_system, xmlNodePtr element_program, xmlNodePtr element_project)
{
    char *args = NULL;
    char *includes = NULL;
    int have_diff = 0;
    xmlNodePtr element_install = NULL;

    int status =0;
    struct s_queue *s_queue_libraries_paths;
    struct s_queue *s_queue_includes_paths;

    struct s_queue *s_queue_libraries;
    struct s_queue *s_queue_includes;
    struct s_queue *s_queue_sources;

    struct s_queue *s_queue_cflags;
    struct s_queue *s_queue_ldflags;

    struct s_queue_hdr *s_walker;

    printf("\n- Build Program: %20s -----------------\n", xmlGetProp(element_program, "name"));

    s_queue_includes_paths = get_includes_paths(doc, element_system->xmlChildrenNode);
    s_queue_libraries_paths = get_libraries_paths(doc, element_system->xmlChildrenNode);

    printf("\n-------------------------------------------------------\n");
    printf("Searching...\n\n");

    s_queue_includes  = get_includes(doc, s_queue_includes_paths, element_program->xmlChildrenNode, &status);
    if( status == 0 ){
	return 0;
    }
    s_queue_libraries = get_libraries(doc, s_queue_libraries_paths, element_program->xmlChildrenNode, &status);
    if( status == 0 ){
	return 0;
    }

    s_queue_sources = get_sources(doc, element_program->xmlChildrenNode);

    s_queue_cflags = get_cflags(doc, element_program->xmlChildrenNode);
    s_queue_ldflags = get_ldflags(doc, element_program->xmlChildrenNode);

    printf("\n");
    printf("Compiling...\n\n");


/* FLAGS */
    if (s_queue_cflags) {
	s_walker = s_queue_cflags->s_first;
	while (s_walker) {
	    if (s_walker->name) {
		includes = str_add(includes, s_walker->name);
		includes = str_add(includes, "  ");
	    }
	    s_walker = s_walker->s_right;
	}
    }
/* INCLUDES */
    if (s_queue_includes) {
	s_walker = s_queue_includes->s_first;
	while (s_walker) {
	    if (s_walker->directory) {
		includes = str_add(includes, "-I");
		includes = str_add(includes, s_walker->directory);
		includes = str_add(includes, "  ");
	    }
	    s_walker = s_walker->s_right;
	}
    }
    if( !file_exist(NULL, xmlGetProp(element_program, "name")) ){
	have_diff = 1;
    }
/* COMPILE */
    if (s_queue_sources) {
	s_walker = s_queue_sources->s_first;
	while (s_walker) {
	    if (file_diff(s_walker->name, xmlGetProp(element_program, "source_path"))) {
		have_diff = 1;
		args = str_add(args, CC);
		args = str_add(args, " -c  ");
		args = str_add(args, includes);
		args = str_add(args, "  ");
		args = str_add(args, xmlGetProp(element_program, "source_path"));
		args = str_add(args, "/");
		args = str_add(args, s_walker->name);
		args = str_add(args, "  -o ");
		args = str_add(args, xmlGetProp(element_program, "source_path"));
		args = str_add(args, "/");
		args = str_add(args, replace_object(s_walker->name));

		printf("Build %30s", s_walker->name);
		fflush(stdout);
		if( run_command(args) ){
			printf("       OK\n");
			fflush(stdout);
		} else {
			printf("       FAILED\n");
			fflush(stdout);

			if( xmlGetProp(element_program, "type") ){
				if( strcasecmp( xmlGetProp(element_project, "type"),"required") == 0 ){
					exit( 0 );
				} else {
					return ;
				}
			}
		}

		clone_tm(s_walker->name, xmlGetProp(element_program, "source_path"));

		free(args);
		args = NULL;
	    } else {
		printf("Build %30s", s_walker->name);
		fflush(stdout);
		printf("       OK - NOT CHANGED\n");
		fflush(stdout);
	    }
	    s_walker = s_walker->s_right;
	}

	args = str_add(args, CC);

	if (have_diff) {
	    /* LDFLAGS */
	    if (s_queue_ldflags) {
		s_walker = s_queue_ldflags->s_first;
		while (s_walker) {
		    if (s_walker->name) {
			args = str_add(args, s_walker->name);
			args = str_add(args, "  ");
		    }
		    s_walker = s_walker->s_right;
		}
	    }

	    /* LIBS */
	    if (s_queue_libraries) {
		s_walker = s_queue_libraries->s_first;
		while (s_walker) {
		    args = str_add(args, "-L");
		    args = str_add(args, s_walker->directory);
		    args = str_add(args, "  ");

		    args = str_add(args, "-l");
		    args = str_add(args, s_walker->name);
		    args = str_add(args, "  ");

		    s_walker = s_walker->s_right;
		}

	    }
	    /* LINK */
	    if (s_queue_sources) {
		s_walker = s_queue_sources->s_first;
		while (s_walker) {
		    args = str_add(args, xmlGetProp(element_program, "source_path"));
		    args = str_add(args, "/");
		    args = str_add(args, replace_object(s_walker->name));
		    args = str_add(args, "  ");

		    s_walker = s_walker->s_right;
		}
		args = str_add(args, " -o ");
		args = str_add(args, xmlGetProp(element_program, "name"));
		args = str_add(args, "  ");

		printf("%s\n", args);



		if( run_command(args) ){
			printf("       OK\n");
			fflush(stdout);
		} else {
			printf("       FAILED\n");
			fflush(stdout);

			if( xmlGetProp(element_program, "type") ){
				if( strcasecmp( xmlGetProp(element_project, "type"),"required") == 0 ){
					exit( 0 );
				} else {
					return ;
				}
			}
		}





	    }
	}
    }
    printf("\n");
    printf("Installing...\n\n");

    free(args);
    args = NULL;

    element_install = getElement(doc, element_program->xmlChildrenNode, "install");
    if (element_install) {
	makedir(xmlGetProp(element_project, "default_prefix"), xmlGetProp(element_install, "directory"));

	args = str_add(args, "cp -f ");
	args = str_add(args, xmlGetProp(element_program, "name"));
	args = str_add(args, " ");
	if (xmlGetProp(element_install, "directory")[0] != '/') {
	    args = str_add(args, xmlGetProp(element_project, "default_prefix"));
	    args = str_add(args, "/");
	}
	args = str_add(args, xmlGetProp(element_install, "directory"));
	printf("%s\n", args);
	run_command(args);

	free(args);
	args = NULL;
    }
    return 1;
}
int getTypeFromString(char *string)
{
    if (strcasecmp(string, "required") == 0) {
	return TYPE_REQUIRED;
    }
    if (strcasecmp(string, "optional") == 0) {
	return TYPE_OPTIONAL;
    }
    if (strcasecmp(string, "secondary") == 0) {
	return TYPE_SECONDARY;
    }
    return 0;
}



struct s_queue *add_queue(struct s_queue *s_q, char *name, char *directory)
{
    struct s_queue_hdr *s_fd;

    if (!s_q) {
	s_q = (struct s_queue *) calloc(1, sizeof(struct s_queue));
    }
    s_fd = (struct s_queue_hdr *) calloc(1, sizeof(struct s_queue_hdr));

    if (!s_q->s_first) {
	s_q->s_first = s_fd;
    } else {
	s_fd->s_right = NULL;
	s_fd->s_left = s_q->s_last;
	s_q->s_last->s_right = s_fd;
    }
    s_q->s_last = s_fd;

    if (directory) {
	strncpy(s_fd->directory, directory, 255);
    }
    if (name) {
	strncpy(s_fd->name, name, 255);
    }
    return s_q;
}

int run_command(char *command)
{
    int result = 0;
    int readed = 0;
    char buffer[4096];
    FILE *fp;

    snprintf(buffer, 4096, "%s 2>&1", command);

    if (!(fp = popen(buffer, "r"))) {
	perror(command);
	exit(0);
    }
    buffer[0] = 0x00;

    readed = fread(buffer, 1, 4096, fp);

    buffer[readed] = 0x00;

    result = pclose(fp);

    if (result != 0) {
	printf("\n\n%s\n\n", buffer);
//	exit(0);
	return 0;
    }
    return 1;
}
