#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "FastMake.h"
#include "xml.h"
#include "search.h"


struct s_queue *get_libraries(xmlDocPtr doc, struct s_queue *s_queue_libraries, xmlNodePtr element_program, int * status )
{
    int type=0;
    char location[255];
    struct s_queue *s_q = NULL;
    int search_result = 0;

    xmlNodePtr element_libraries = NULL;
    xmlNodePtr element_name = NULL;

    *status = 1;

    if (!element_program) {
	printf("Missing Program Element\n");
	exit(0);
    }
    element_libraries = getElement(doc, element_program, "libraries")->xmlChildrenNode;

    element_name = element_libraries;
    while ((element_name = getElement(doc, element_name, "name"))) {
	if ((search_result = search_library_file(xmlNodeListGetString(doc, element_name->xmlChildrenNode, 1), s_queue_libraries, location))) {

	    printf("[Library: %20s] [FOUND]: %30s\n", xmlNodeListGetString(doc, element_name->xmlChildrenNode, 1), location);

	    s_q = add_queue(s_q, xmlNodeListGetString(doc, element_name->xmlChildrenNode, 1), location);
	} else {
	    type = getTypeFromString((char *) xmlGetProp(element_name, "type"));
	    switch(type) {
	    	case TYPE_REQUIRED:
			printf("[Library: %20s] [NOT FOUND - CRITICAL]\n", xmlNodeListGetString(doc, element_name->xmlChildrenNode, 1));
			exit(0);
		break;
		case TYPE_SECONDARY:
			printf("[Library: %20s] [NOT FOUND - ABORT]\n", xmlNodeListGetString(doc, element_name->xmlChildrenNode, 1));
			*status = 0;
			return NULL;
		break;
		case TYPE_OPTIONAL:
			printf("[Library: %20s] [NOT FOUND - NON-CRITICAL\n", xmlNodeListGetString(doc, element_name->xmlChildrenNode, 1));
		break;
	    }
	}
	element_name = element_name->next;
    }
    return s_q;
}

struct s_queue *get_includes(xmlDocPtr doc, struct s_queue *s_queue_includes, xmlNodePtr element_program, int *status)
{
    int type=0;
    char location[255];
    struct s_queue *s_q = NULL;

    xmlNodePtr element_includes = NULL;
    xmlNodePtr element_filename = NULL;

    *status = 1;

    if (!element_program) {
	printf("Missing Program Element\n");
	exit(0);
    }
    element_includes = getElement(doc, element_program, "includes")->xmlChildrenNode;

    element_filename = element_includes;
    while ((element_filename = getElement(doc, element_filename, "filename"))) {
	if (search_include_file(xmlNodeListGetString(doc, element_filename->xmlChildrenNode, 1), s_queue_includes, location)) {
	    printf("[Include: %20s] [FOUND]: %30s\n", xmlNodeListGetString(doc, element_filename->xmlChildrenNode, 1), location);

	    s_q = add_queue(s_q, xmlNodeListGetString(doc, element_filename->xmlChildrenNode, 1), location);
	} else {
	    type = getTypeFromString((char *) xmlGetProp(element_filename, "type"));
	    switch(type) {
	    	case TYPE_REQUIRED:
	    	    printf("[Include: %20s] [NOT FOUND - CRITICAL]\n", xmlNodeListGetString(doc, element_filename->xmlChildrenNode, 1));
	    	    exit(0);
		case TYPE_SECONDARY:
			printf("Include: %20s] [NOT FOUND - BREAK]\n", xmlNodeListGetString(doc, element_filename->xmlChildrenNode, 1));
			*status = 0;
			return NULL;
		break;
		case TYPE_OPTIONAL:
	    	    printf("[Include: %20s] [NOT FOUND - NON-CRITICAL\n", xmlNodeListGetString(doc, element_filename->xmlChildrenNode, 1));
		break;
	    }
	}

	element_filename = element_filename->next;
    }
    return s_q;
}
struct s_queue *get_includes_paths(xmlDocPtr doc, xmlNodePtr element_system)
{
    struct s_queue *s_q = NULL;

    xmlNodePtr element_includes = NULL;
    xmlNodePtr element_path = NULL;

    if (!element_system) {
	printf("Missing Program Element\n");
	exit(0);
    }
    element_includes = getElement(doc, element_system, "includes")->xmlChildrenNode;

    element_path = element_includes;
    while ((element_path = getElement(doc, element_path, "path"))) {
	if (file_exist(NULL, xmlNodeListGetString(doc, element_path->xmlChildrenNode, 1))) {
	    printf("[System Include path: %20s]\n", xmlNodeListGetString(doc, element_path->xmlChildrenNode, 1));

	    s_q = add_queue(s_q, xmlNodeListGetString(doc, element_path->xmlChildrenNode, 1), xmlNodeListGetString(doc, element_path->xmlChildrenNode, 1));
	}

	element_path = element_path->next;
    }

    return s_q;
}
struct s_queue *get_libraries_paths(xmlDocPtr doc, xmlNodePtr element_system)
{
    struct s_queue *s_q = NULL;

    xmlNodePtr element_libraries = NULL;
    xmlNodePtr element_path = NULL;

    if (!element_system) {
	printf("Missing Program Element\n");
	exit(0);
    }
    element_libraries = getElement(doc, element_system, "libraries")->xmlChildrenNode;

    element_path = element_libraries;
    while ((element_path = getElement(doc, element_path, "path"))) {
	if (file_exist(NULL, xmlNodeListGetString(doc, element_path->xmlChildrenNode, 1))) {
	    printf("[System library path: %20s]\n", xmlNodeListGetString(doc, element_path->xmlChildrenNode, 1));

	    s_q = add_queue(s_q, xmlNodeListGetString(doc, element_path->xmlChildrenNode, 1), xmlNodeListGetString(doc, element_path->xmlChildrenNode, 1));
	}

	element_path = element_path->next;
    }
    return s_q;
}
struct s_queue *get_sources(xmlDocPtr doc, xmlNodePtr element_program)
{
    struct s_queue *s_q = NULL;

    xmlNodePtr element_sources = NULL;
    xmlNodePtr element_filename = NULL;

    element_sources = getElement(doc, element_program, "sources")->xmlChildrenNode;

    element_filename = element_sources;
    while ((element_filename = getElement(doc, element_filename, "filename"))) {
	if (xmlNodeListGetString(doc, element_filename->xmlChildrenNode, 1)) {
	    s_q = add_queue(s_q, xmlNodeListGetString(doc, element_filename->xmlChildrenNode, 1), NULL);
	}
	element_filename = element_filename->next;
    }
    return s_q;
}
struct s_queue *get_cflags(xmlDocPtr doc, xmlNodePtr element_program)
{
    struct s_queue *s_q = NULL;

    xmlNodePtr element_options = NULL;
    xmlNodePtr element_cflags = NULL;

    element_options = getElement(doc, element_program, "options")->xmlChildrenNode;

    element_cflags = element_options;
    while ((element_cflags = getElement(doc, element_cflags, "cflags"))) {
	if (xmlNodeListGetString(doc, element_cflags->xmlChildrenNode, 1)) {
	    s_q = add_queue(s_q, xmlNodeListGetString(doc, element_cflags->xmlChildrenNode, 1), NULL);
	}

	element_cflags = element_cflags->next;
    }
    return s_q;
}
struct s_queue *get_ldflags(xmlDocPtr doc, xmlNodePtr element_program)
{
    struct s_queue *s_q = NULL;

    xmlNodePtr element_options = NULL;
    xmlNodePtr element_ldflags = NULL;

    element_options = getElement(doc, element_program, "options")->xmlChildrenNode;

    element_ldflags = element_options;
    while ((element_ldflags = getElement(doc, element_ldflags, "ldflags"))) {
	if (xmlNodeListGetString(doc, element_ldflags->xmlChildrenNode, 1)) {
	    s_q = add_queue(s_q, xmlNodeListGetString(doc, element_ldflags->xmlChildrenNode, 1), NULL);
	}
	element_ldflags = element_ldflags->next;
    }
    return s_q;
}
int search_library_file(char *name, struct s_queue *s_queue_library, char *location)
{
    int result = 0;
    char filename[255];
    struct s_queue_hdr *s_walker;

    snprintf(filename, 255, "lib%s.so", name);

    if (s_queue_library) {
	s_walker = s_queue_library->s_first;
	while (s_walker) {
	    if (find_file(s_walker->directory, filename, location)) {
		return 1;
	    }
	    s_walker = s_walker->s_right;
	}
    }
    return result;
}
int search_include_file(char *name, struct s_queue *s_queue_include, char *location)
{
    int result = 0;
    struct s_queue_hdr *s_walker;

    if (s_queue_include) {
	s_walker = s_queue_include->s_first;
	while (s_walker) {
	    if (find_file(s_walker->directory, name, location)) {
		return 1;
	    }
	    s_walker = s_walker->s_right;
	}
    }
    return result;
}
