#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <dirent.h>
#include <utime.h>

#define FLAGS ""
#define CC "cc"

#define NAME    "install.bin"

#define LIBRARY_PATHS "/lib","/usr/lib","/usr/local/lib"
#define LIBRARY_PATHS_COUNT 3

#define INCLUDE_PATHS "/usr/local/include", "/usr/include"
#define INCLUDE_PATHS_COUNT 2

#define LIBRARIES "xml2"
#define LIBRARIES_COUNT 1

#define INCLUDES "iconv.h","libxml/xmlversion.h"
#define INCLUDES_COUNT 2

#define SOURCES ".fastmake/FastMake",".fastmake/file",".fastmake/search",".fastmake/string",".fastmake/xml"
#define SOURCES_COUNT 5


struct s_queue_hdr {
        char directory[255];
        char name     [255];

        struct s_queue_hdr * s_right;
        struct s_queue_hdr * s_left;
};
struct s_queue {
        struct s_queue_hdr * s_first;
        struct s_queue_hdr * s_last;
};

int file_exist( char * name, char * path );
struct s_queue  * add_queue( struct s_queue  * s_q, char *name, char * directory  );
struct s_queue  * search_library();
struct s_queue  * search_include(int * status);
int file_type( char * name, char * path );
int show_queue(   struct s_queue * s_q );
int file_diff(char *file);
int clone_tm(char *file);

char *library_paths   []  ={ LIBRARY_PATHS };
char *include_paths   []  ={ INCLUDE_PATHS };
char *libraries[] ={ LIBRARIES };
char *includes[] ={ INCLUDES };
char *sources[] ={ SOURCES };

int main()
{
       int i;
       int status=0;
       struct s_queue * s_queue_lib;
       struct s_queue * s_queue_include;
      
       
        s_queue_lib                 = search_library( &status );
        if( status )
                return -1;
         s_queue_include         = search_include( &status );        

        if( status )
                return -1;
                
       show_queue(   s_queue_lib );
       show_queue(   s_queue_include );
       
       compile( s_queue_lib, s_queue_include );
       return 0;
       
}
int compile(   struct s_queue * s_qlib ,struct s_queue * s_qinclude )
{
      int i;
       char line[4096];
       char buffer[255];
       int changed=0;
       
       
       struct s_queue_hdr * s_walk;    

       line[0]=0x00;
       
       
       
       for( i=0;i<SOURCES_COUNT;i++ ){
         if (file_diff( sources[i] )){
               changed = 1;
               snprintf( line, 255, "%s -c %s  ", CC, FLAGS );
               strcat( line, sources[i] );         
               strcat( line, ".c  -o" );         
               strcat( line, sources[i] );         
               strcat( line, ".o " );         

               s_walk = s_qinclude->s_first;       
               while( s_walk ){
                        snprintf( buffer, 255, "-I%s ", s_walk->directory );
                        
                        strcat( line, buffer );
                        
                        s_walk = s_walk->s_right;
               }
               printf("%s\n", line      );
               
               system( line );
               
               clone_tm(sources[i] );
         }
       }
       if( changed ){
               snprintf( line, 255, "%s %s -o %s  ", CC, FLAGS,NAME );
               for( i=0;i<SOURCES_COUNT;i++ ){
                       strcat( line, sources[i] );         
                       strcat( line, ".o " );         
               }
               
               s_walk = s_qlib->s_first;       
               while( s_walk ){
                        snprintf( buffer, 255, "-L%s ", s_walk->directory );
                        
                        strcat( line, buffer );
                        
                        s_walk = s_walk->s_right;
               }
               
               s_walk = s_qlib->s_first;       
               while( s_walk ){
                        snprintf( buffer, 255, "-l%s ", s_walk->name );
                        
                        strcat( line, buffer );
                        
                        s_walk = s_walk->s_right;
               }

               printf("%s\n", line      );
               
               system( line );
       }
       return 1;
}

int show_queue(   struct s_queue * s_q )
{
       struct s_queue_hdr * s_walk;    
       
       s_walk = s_q->s_first;
       
       while( s_walk ){
                
                s_walk = s_walk->s_right;
       }
       return 1;
}
struct s_queue * search_library(int * status)
{
       int i;
       char location[255];
       char filename[255];
       
       struct s_queue * s_queue_lib = NULL;

       for( i=0;i<LIBRARIES_COUNT;i++ ){
               printf("Searching Library: [%20s]        " , libraries[i] );
               
               snprintf( filename,255, "lib%s.so", libraries[i] );
               
               if( search_library_file( filename, location) ){
                       s_queue_lib = add_queue( s_queue_lib, libraries[i], location );
                       
                      printf("FOUND: %s\n", location );
                       
               } else {
                        printf("NOT FOUND\n" );

                        printf("Missing library: %s\n", libraries[i] );
                        printf("Exiting\n" );
                        
                        *status = 1;
                        return NULL;
               }
               fflush(stdout);               
       }
       return s_queue_lib;
}
struct s_queue * search_include(int * status)
{
       int i;
       char location[255];
       struct s_queue * s_queue_lib = NULL;

       for( i=0;i<INCLUDES_COUNT;i++ ){
               printf("Searching Include: [%20s]        " , includes[i] );
               
               if( search_include_file( includes[i], location) ){
                       s_queue_lib = add_queue( s_queue_lib,  includes[i], location );
                       
                      printf("FOUND: %s\n", location );
                       
               } else {
                        printf("NOT FOUND\n" );

                        printf("Missing include: %s\n", includes[i] );
                        printf("Exiting\n" );
                        
                        *status = 1;
                        return NULL;
               }
               fflush(stdout);
       }
       return s_queue_lib;
}

struct s_queue  * add_queue( struct s_queue  * s_q, char *name, char * directory  ) {
        struct s_queue_hdr * s_fd;

        if( !s_q ){
                s_q = ( struct s_queue  * )calloc( 1, sizeof( struct s_queue  ) );
        }
        s_fd = ( struct s_queue_hdr * )calloc( 1, sizeof( struct s_queue_hdr ) );

        if( !s_q->s_first ){
                s_q->s_first = s_fd;
        } else {
                s_fd->s_right     = NULL;
                s_fd->s_left      = s_q->s_last;
                s_q->s_last->s_right   = s_fd;
        }
        s_q->s_last = s_fd;

        strncpy( s_fd->directory, directory, 255 );
        strncpy( s_fd->name    , name     , 255 );        
        return s_q;
}

int search_library_file( char * name, char * location )
{
        DIR *sDir;
        int result = 0;
        int i;

        for( i=0;i< LIBRARY_PATHS_COUNT;i++){
                if(  find_file( library_paths[i], name, location ) ){
                        return 1;
                }
        }      
        return result;
}
int search_include_file( char * name, char * location )
{
        DIR *sDir;
        int result = 0;
        int i;

        for( i=0;i< INCLUDE_PATHS_COUNT;i++){
                if(  find_file( include_paths[i], name, location ) ){
                        return 1;
                }
        }      
        return result;
}
int find_file( char * directory, char * filename, char * result )
{
       struct dirent *sDirent;
      
        DIR *sDir;
        int i;
        
        char fullpath[255];
        
         snprintf( fullpath, 255, "%s/%s", directory, filename  );
         if( file_exist( filename,directory ) ){
                 strncpy( result, directory, 255 );
                 return 1;
         }
         if ((sDir = opendir( directory ))) {
                   while ((sDirent = readdir(   sDir))) {
                         if( strcmp( sDirent->d_name,"." ) && strcmp( sDirent->d_name, ".." )&& strcmp( sDirent->d_name, "modules" ) ){
                                 snprintf( fullpath, 255, "%s/%s", directory, sDirent->d_name  );
                                 if( S_ISDIR( file_type( sDirent->d_name, directory)) ){
                                         if( find_file( fullpath, filename, result ) ){
                                                return 1;
                                         }
                                 }
                         }
                   }
                   closedir( sDir );
         }
        return 0;
}
int file_exist( char * name, char * path )
{
        struct stat st;      
        char filename[255];
        
        snprintf( filename, 255, "%s/%s", path, name );
        
        if (stat(filename, (struct stat *) &st) != 0) {
                return 0;
        }
        return 1;
}
int file_type( char * name, char * path )
{
        struct stat st;      
        char filename[255];
        
        snprintf( filename, 255, "%s/%s", path, name );
        
        if (lstat(filename, (struct stat *) &st) != 0) {
                return 0;
        }
        return st.st_mode;
}
/* Verify modify time */
int file_diff(char *file )
{
    struct stat source;
    struct stat object;
    char filename_source[255];
    char filename_dest[255];
    
    snprintf( filename_source, 255, "%s.c" , file );
    snprintf( filename_dest, 255, "%s.o",file );
    
    stat(filename_source, &source);
    stat(filename_dest, &object);

    if (source.st_mtime != object.st_mtime) {
          return 1;
    }
    return 0;
}
/* Copy modify time */
int clone_tm(char *file )
{
    char filename[255];
    struct stat source;
    struct utimbuf ut;
    char filename_source[255];
    char filename_dest[255];

    snprintf( filename_source, 255, "%s.c" , file );
    snprintf( filename_dest, 255, "%s.o",file );

    stat(filename_source, &source);

    ut.actime = source.st_mtime;
    ut.modtime = source.st_mtime;

    utime(filename_dest, &ut);

    return 1;
}
