/***************************************************************************
    begin                : Sun Oct 05
    copyright            : (C) 2003 by Fabio Yasusi Yamamoto
    email                : fabio@hostname.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <time.h>
#include <sys/un.h>
#include <sys/errno.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <time.h>
#include <arpa/inet.h>
#include <fcntl.h>

#include "log.h"


struct s_blog_hdr  * blog_init( char * filename_data, char * filename_idx, int flags )
{
      struct s_blog_hdr * s_blog;
      int file_size;
      
      s_blog = calloc( 1, sizeof( struct s_blog_hdr ));
      
      if( !s_blog ){
                 perror( "calloc");
                 exit   ( 0 );
      }
      
      s_blog->data_fd = open( filename_data, flags );      
      if( s_blog->data_fd == -1 ){
      	    free( s_blog );
    	    return NULL;
      }  

      s_blog->idx_fd = open( filename_idx, flags );      
      if( s_blog->idx_fd == -1 ){
      	    free( s_blog );
    	    close( s_blog->data_fd );
    	    return NULL;
      }  

    if((s_blog->data_size = get_file_size(filename_data))<=0){
      return 0;
    }
    s_blog->data_map = mmap(0, s_blog->data_size, PROT_READ, MAP_SHARED, s_blog->data_fd, 0);

    if((s_blog->idx_size = get_file_size(filename_idx))<=0){
      return 0;
    }
    s_blog->idx_map = mmap(0, s_blog->idx_size, PROT_READ, MAP_SHARED, s_blog->idx_fd, 0);

    return s_blog;
}
void blog_close( struct s_blog_hdr * s_blog )
{
    if( !s_blog ){
	return ;	
    }
    munmap( s_blog->data_map, s_blog->data_size );
    munmap( s_blog->idx_map, s_blog->idx_size );
    close( s_blog->data_fd );
    close( s_blog->idx_fd );
    
    free( s_blog );
}
void blog_add( char *filename_log, char * filename_index,char * user, unsigned long address, char action )
{
    int 	fd;
    int pad=0;
    struct s_blog_data s_data;

    unsigned long user_id;
    
    if( get_file_size( filename_log ) <=0 ){
	pad = 1;
    }

    if(( fd = open( filename_log, O_RDWR|O_APPEND|O_CREAT,0666 ))==-1){
        perror( "open" );
        return ;
    }

    user_id = blog_have_user( filename_index, user );
    
    if( !user_id ){
	user_id = blog_insert_user( filename_index, user );
    }
    s_data.time 	= time(NULL);
    s_data.user_id 	= user_id;
    s_data.action  	= action;
    s_data.address      = address;

    if( pad ){
	write( fd, &s_data, 1);    
    }
    write( fd, &s_data, sizeof( struct s_blog_data ));

    close( fd );
    
}
int blog_insert_user( char * filename, char * user )
{
    int i;
    int fd;
    int size;
    int pos=0;
    char pad=0;
    
    if( get_file_size( filename ) <=0 ){
	pad = 1;
    }
    if ((fd = open(filename, O_RDWR|O_APPEND|O_CREAT, 0666)) <= 0) {
	return 0;
    }

    pos = lseek( fd, 0, SEEK_END );

    size = strlen( user );

    if( size > 255 ){
	return 0;
    }
    if( pad ){
	write( fd, &size, 1 );
    }
    pos = lseek( fd, 0, SEEK_END );
    
    write( fd, &size, 1 );
    write( fd, user, size + 1 );

    close( fd );
    
    return pos;
}



int blog_have_user( char * filename, char * user )
{
    int i;
    int fd;
    int file_size;
    char size;
    char * map;
    char buffer[255]; 
    
    if((file_size = get_file_size(filename))<=0){
	return 0;
    }
    if ((fd = open(filename, O_RDONLY)) <= 0) {
	return 0;
    }
    map = mmap(0, file_size, PROT_READ, MAP_SHARED, fd, 0);

    i = 1;
    while( i< file_size ){
	size = map[i];
	if( strcmp( user, &map[i+1]) == 0 ){
	    return i;
	}
	i += size;
    }
    munmap(map, file_size);

    close( fd );
    
    return 0;
}
char * blog_get_user( struct s_blog_hdr * s_blog, int pos )
{
    int i;
    int fd;
    int file_size;
    char size;
    char * map;
    char * result=NULL;
    

    if( pos>=s_blog->idx_size || pos == 0){
	return NULL;
    }
    
    size = s_blog->idx_map[pos];
    
    result = calloc( 1, size + 1 );

    memcpy( result, &s_blog->idx_map[pos+1], size );
        
    return result;
}


int blog_search_interval( struct s_blog_hdr * s_blog, int tv, unsigned long address )
{
    int i;
    int size;
    int struct_size;
    
    int last_value=0;
    struct s_blog_data * s_data;

    if( !s_blog ){
    	return 0;
    }
    struct_size = sizeof( struct s_blog_data );

    for(i=1;i<s_blog->data_size;i=i+struct_size){
	s_data = ( struct s_blog_data * )&s_blog->data_map[i];
	
	if( s_data->address == address && s_data->action == ACTION_GRANTED && s_data->time < tv ){
	    last_value = i;
	}	
	if( s_data->address == address && s_data->action == ACTION_GRANTED && s_data->time > tv ){
	    break;
	}
	if( s_data->address == address && s_data->action == ACTION_REVOKE && s_data->time > tv ){
	    break;
	}
    }
    
   if(  last_value !=0 ){
            s_data = ( struct s_blog_data * )&s_blog->data_map[last_value];
            
            last_value = s_data->user_id;
   }            
    return last_value;
}
