/***************************************************************************
                          list.c  -  description
                             -------------------
    begin                : Thu May 1 2002
    copyright            : (C) 2002 by Fabio Yasusi Yamamoto
    email                : fabio@hostname.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <openssl/ssl.h>

#include "html.h"
#include "file.h"
#include "util.h"



char *html_parse_arg(char *string, char sep, int pos);
char *execute_popen(char *filename);
int template_system(struct s_template *s_html);

void template_close(struct s_template *s_html)
{
    if (s_html) {
	if (s_html->html) {
	    free(s_html->html);
	}
	free(s_html);
    }
}
struct s_template *template_init(char *filename)
{

    int fd;
    struct s_template *s_html;

    if ((s_html = (struct s_template *) calloc(1,sizeof(struct s_template))) == NULL) {
	return NULL;
    }
    if ((s_html->size = get_file_size(filename)) <= 0) {
	printf("File not found: %s\n", filename );
	fflush( stdout );
	free( s_html );
	return NULL;
    }
    if ((fd = open(filename, O_RDONLY)) <= 0) {
	printf("File not found: %s\n", filename );
	fflush( stdout );
	return NULL;
    }
    if ((s_html->html = (char *) malloc(s_html->size + 1)) == NULL) {
	exit(0);
    }
    if (read(fd, s_html->html, s_html->size) == -1) {
	close(fd);
	free(s_html);
	return NULL;
    }
    s_html->html[s_html->size] = 0x00;
    close(fd);

    template_system(s_html);

    return s_html;
}
int template_include(struct s_template *s_html, char *name, char *filename)
{
    int fd;
    char *memory;
    int size;

    size = get_file_size(filename);

    if ((fd = open(filename, O_RDONLY)) <= 0) {
	return 0;
    }
    memory = mmap(0, size, PROT_READ, MAP_SHARED, fd, 0);
    close(fd);

    template_write(s_html, name, memory);

    munmap(memory, size);

    return 1;
}
int html_showfile_block(char *filename)
{
    int fd;
    char *map;
    int size;

    if (strstr(filename, "..")) {
	printf("SORRY ;)\n");
	return 0;
    }
    if (!checkstr(filename, "ABCDEFGHIJKLMNOPQRSTUVWXYZ- +=.12345()[]#$67890:_")) {
	return 0;
    }

    size = get_file_size(filename);

    if ((fd = open(filename, O_RDONLY)) <= 0) {
	return 0;
    }

    map = mmap(0, size, PROT_READ, MAP_SHARED, fd, 0);
    write(1, map, size);
    munmap(map, size);

    return 1;
}
int template_write(struct s_template *s_html, char *name, char *value)
{
    int i;
    int ok = 0;
    int name_size;
    int value_size;

    char *result = NULL;

    if (!name || !value || !s_html) {
	return 0;
    }
    name_size = strlen(name);
    value_size = strlen(value);

    for (i = 0; i < s_html->size; i++) {
	if (s_html->html[i] == '[' && strncasecmp(&s_html->html[i + 1], name, name_size) == 0 && s_html->html[i + 1 + name_size] == ']') {
	    ok = 1;
	    fflush(stdout);
	    if ((result = malloc(s_html->size + (value_size - name_size - 2) + 1)) == NULL) {
		printf("internal error - memory\n");
		exit(0);
	    }

	    memcpy(result, s_html->html, i);
	    memcpy(&result[i], value, value_size);

	    memcpy(&result[i + value_size], &s_html->html[i + name_size + 2], s_html->size - i - 2 - name_size);


	    free(s_html->html);
	    s_html->html = result;
	    s_html->html[s_html->size + (value_size - name_size - 2) ] = 0x00;
	    s_html->size = s_html->size + (value_size - name_size - 2);
	}
    }
    return ok;
}
int template_n_write(struct s_template *s_html, char *name, char *value, int value_size)
{
    int i;
    int ok = 0;
    int name_size;

    char *result = NULL;

    if (!name || !value || !s_html) {
	return 0;
    }
    name_size = strlen(name);
    value_size = strlen(value);

    for (i = 0; i < s_html->size; i++) {
	if (s_html->html[i] == '[' && strncasecmp(&s_html->html[i + 1], name, name_size) == 0 && s_html->html[i + 1 + name_size] == ']') {
	    ok = 1;
	    fflush(stdout);
	    if ((result = malloc(s_html->size + (value_size - name_size - 2) + 1)) == NULL) {
		printf("internal error - memory\n");
		exit(0);
	    }

	    memcpy(result, s_html->html, i);
	    memcpy(&result[i], value, value_size);

	    memcpy(&result[i + value_size], &s_html->html[i + name_size + 2], s_html->size - i - 2 - name_size);


	    free(s_html->html);
	    s_html->html = result;
	    s_html->html[s_html->size + (value_size - name_size - 2) ] = 0x00;
	    s_html->size = s_html->size + (value_size - name_size - 2);
	}
    }
    return ok;
}
int template_printf(struct s_template *s_html, char *name, char *value, ...)
{
    int i;
    int ok = 0;
    int name_size;
    int temp01_size;

    char *result;
    char temp01[8192];
    va_list args;

    va_start(args, value);
    vsnprintf(temp01, 8192, value, args);
    va_end(args);

    name_size = strlen(name);
    temp01_size = strlen(temp01);

    for (i = 0; i < s_html->size; i++) {
	if (s_html->html[i] == '[' && strncasecmp(&s_html->html[i + 1], name, name_size) == 0 && s_html->html[i + 1 + name_size] == ']') {
	    ok = 1;
	    result = malloc(s_html->size + (temp01_size - name_size - 2) + 1);
	    memcpy(result, s_html->html, i);
	    memcpy(&result[i], temp01, temp01_size);
	    memcpy(&result[i + temp01_size], &s_html->html[i + name_size + 2], s_html->size - i - 2 - name_size);


	    free(s_html->html);
	    s_html->html = result;
	    s_html->html[s_html->size + (temp01_size - name_size - 2) ] = 0x00;
	    s_html->size = s_html->size + (temp01_size - name_size - 2);
	}
    }
    return ok;
}

void template_show(struct s_template *s_html)
{
    if (s_html) {
	if (s_html->html) {
	    write(1, s_html->html, s_html->size);
	}
    }
}
void template_show_fd(struct s_template *s_html, int fd, SSL * ssl_client)
{
    if (s_html) {
	if (s_html->html) {
	    if( ssl_client ){
	    	SSL_write( ssl_client, s_html->html, s_html->size);
	    } else {
	    	write(fd, s_html->html, s_html->size);
	    }
	}
    }
}
int template_system(struct s_template *s_html)
{
    int i;
    int z;

    int ok = 0;
    char name[255];
    char *value;
    char *argv[64];
    int argc;
    int name_size;
    int value_size;
    int f;
    char *result = NULL;
    void *module;

    if (!s_html) {
	return 0;
    }
    for (i = 0; i < s_html->size; i++) {
	/* Search for '[' */
	if (s_html->html[i] == '[') {
	    /* Search for '$' */
	    if (s_html->html[i + 1] == '$') {
		for (f = i + 2; f < s_html->size; f++) {
		    if (s_html->html[f] == ']') {
			break;
		    }
		}
		memcpy(name, &s_html->html[i + 2], f - i - 2);
		name[f - i - 2] = 0x00;
		name_size = f - i - 2;

		if ((value = getenv(name))) {
		    value_size = strlen(value);
		} else {
		    value_size = 0;
		}

		ok = 1;
		fflush(stdout);
		if ((result = malloc(s_html->size + (value_size - name_size - 2) + 1)) == NULL) {
		    printf("internal error - memory\n");
		    exit(0);
		}
		memcpy(result, s_html->html, i);
		memcpy(&result[i], value, value_size);
		memcpy(&result[i + value_size], &s_html->html[i + name_size + 3], s_html->size - i - 3 - name_size);

		free(s_html->html);
		s_html->html = result;
		s_html->html[s_html->size + (value_size - name_size - 3) + 1] = 0x00;
		s_html->size = s_html->size + (value_size - name_size - 3);
	    }
	    /* Search for 'DL:' */
/*	    else if (s_html->size > i + 4 && s_html->html[i + 1] == 'D' && s_html->html[i + 2] == 'L' && s_html->html[i + 3] == ':') {
		for (f = i + 4; f < s_html->size; f++) {
		    if (s_html->html[f] == ']') {
			break;
		    }
		}

		memcpy(name, &s_html->html[i + 4], f - i - 4);
		name[f - i - 4] = 0x00;
		name_size = f - i - 4;
		memset(argv, 0x00, 255);
		for (z = 0; z < 64; z++) {
		    if (!(argv[z] = html_parse_arg(name, ':', z))) {
			break;
		    }
		}
		argc = i;
		module = html_load_module(argv[0]);
		if (!(value = cgi_main(argc, argv))) {
		    html_close_module(module);
		    break;
		}
		html_close_module(module);

		for (z = 0; z < 64; z++) {
		    if (argv[z]) {
			free(argv[z]);
		    } else {
			break;
		    }
		}
		value_size = strlen(value);

		ok = 1;
		fflush(stdout);
		if ((result = malloc(s_html->size + (value_size - name_size - 4) + 1)) == NULL) {
		    printf("internal error - memory\n");
		    exit(0);
		}
		memcpy(result, s_html->html, i);
		memcpy(&result[i], value, value_size);

		memcpy(&result[i + value_size], &s_html->html[i + name_size + 5], s_html->size - i - 5 - name_size);


		free(s_html->html);
		s_html->html = result;
		s_html->html[s_html->size + (value_size - name_size - 5) + 1] = 0x00;
		s_html->size = s_html->size + (value_size - name_size - 5);
		free(value);
*/	     else if (s_html->size > i + 6 && s_html->html[i + 1] == 'E' && s_html->html[i + 2] == 'X' && s_html->html[i + 3] == 'E' && s_html->html[i + 4] == 'C' && s_html->html[i + 5] == ':') {
		for (f = i + 6; f < s_html->size; f++) {
		    if (s_html->html[f] == ']') {
			break;
		    }
		}

		memcpy(name, &s_html->html[i + 6], f - i - 6);
		name[f - i - 6] = 0x00;
		name_size = f - i - 6;


		value = execute_popen(name);
		value_size = strlen(value);

		ok = 1;
		fflush(stdout);
		if ((result = malloc(s_html->size + (value_size - name_size - 6) + 1)) == NULL) {
		    printf("internal error - memory\n");
		    exit(0);
		}
		memcpy(result, s_html->html, i);
		memcpy(&result[i], value, value_size);

		memcpy(&result[i + value_size], &s_html->html[i + name_size + 7], s_html->size - i - 7 - name_size);


		free(s_html->html);
		s_html->html = result;
		s_html->html[s_html->size + (value_size - name_size - 3) + 1] = 0x00;
		s_html->size = s_html->size + (value_size - name_size - 3);
		free(value);
	    /* Search for 'SHOW:' */
	    } else if (s_html->size > i + 6 && s_html->html[i + 1] == 'S' && s_html->html[i + 2] == 'H' && s_html->html[i + 3] == 'O' && s_html->html[i + 4] == 'W' && s_html->html[i + 5] == ':') {
		/* Search for ']' */		    
		for (f = i + 6; f < s_html->size; f++) {
		    if (s_html->html[f] == ']') {
			break;
		    }
		}
		
		memcpy(name, &s_html->html[i + 6], f - i - 6);
		name[f - i - 6] = 0x00;
		name_size = f - i - 6;

		value = file_content(name);
		value_size = strlen(value);

		ok = 1;
		fflush(stdout);
		if ((result = malloc(s_html->size + (value_size - name_size - 6) + 1)) == NULL) {
		    printf("internal error - memory\n");
		    exit(0);
		}
		memcpy(result, s_html->html, i);
		memcpy(&result[i], value, value_size);

		memcpy(&result[i + value_size], &s_html->html[i + name_size + 7], s_html->size - i - 7 - name_size);
		
		result[value_size+s_html->size- 7 - name_size]=0x00;    

		free(s_html->html);
		s_html->html = result;
		s_html->html[s_html->size + (value_size - name_size - 7)] = 0x00;
		s_html->size = s_html->size + (value_size - name_size - 7);
		free(value);
	    }
	}
    }
    return ok;
}
char *html_parse_arg(char *string, char sep, int pos)
{
    int i;
    int z;
    int start;
    int count = 0;
    char *result = NULL;
    int str_size;

    str_size = strlen(string);

    for (i = 0; i < str_size; i++) {
	if (string[i] == sep || i == 0) {
	    if (count == pos) {
		if (i) {
		    i++;
		}
		start = i;
		for (z = i; z < str_size; z++) {
		    if (string[z] == sep || string[z] == ']') {
			result = malloc(255);
			memcpy(result, &string[start], z - i);
			result[z - i] = 0x00;

/*			if (strcmp(result, "$LOGIN") == 0 || strcmp(result, "$ID") == 0 || strcmp(result, "$USER") == 0) {
			    free(result);
			    result = malloc(strlen(g_login) + 1);
			    memcpy(result, g_login, strlen(g_login));
			    result[strlen(g_login)] = 0x00;
			}
*/			return result;
		    }
		}
		result = malloc(255);

		memcpy(result, &string[start], str_size);
/*		if (strcmp(result, "$LOGIN") == 0 || strcmp(result, "$ID") == 0 || strcmp(result, "$USER") == 0) {
		    free(result);
		    result = malloc(strlen(g_login) + 1);
		    memcpy(result, g_login, strlen(g_login));
		    result[strlen(g_login)] = 0x00;
		}
*/
		return result;
	    } else {
		count++;
	    }
	}
    }
    return NULL;
}
char *execute_popen(char *filename)
{
    char *result = NULL;
    char buffer[1024];
    int totlen = 0;
    int size;
    FILE *fp;

    if ((fp = popen(filename, "r")) == NULL) {
	return NULL;
    }

    while (!feof(fp)) {
	if ((size = fread(buffer, 1, 1024, fp)) <= 0) {
	    break;
	}
	if ((result = realloc(result, size + totlen + 1)) == NULL) {
	    return NULL;
	}
	memcpy(&result[totlen], buffer, size);
	totlen += size;
    }
    result[totlen] = 0x00;
    pclose(fp);

    return result;
}
